      SUBROUTINE USLAG2 &
!     -------------------------------------------------------------
      ( IDBIA0 , IDBRA0 ,&
        NDIM   , NCELET , NCEL   , NFAC   , NFABOR , NFML   , NPRFML ,&
        NNOD   , LNDFAC , LNDFBR , NCELBR ,                           &
        NVAR   , NSCAL  , NPHAS  ,                                    &
        NBPMAX , NVP    , NVP1   , NVEP   , NIVEP  ,                  &
        NTERSL , NVLSTA , NVISBR ,                                    &
        NIDEVE , NRDEVE , NITUSE , NRTUSE ,                           &
        IFACEL , IFABOR , IFMFBR , IFMCEL , IPRFML , MAXELT , LSTELT ,&
        IPNFAC , NODFAC , IPNFBR , NODFBR ,                           &
        ITYPFB , ITRIFB , ITEPA  , IFRLAG ,                           &
        IDEVEL , ITUSER , IA     ,                                    &
        XYZCEN , SURFAC , SURFBO , CDGFAC , CDGFBO , XYZNOD , VOLUME ,&
        DT     , RTPA   , PROPCE , PROPFA , PROPFB ,                  &
        COEFA  , COEFB  ,                                             &
        ETTP   , TEPA   ,                                             &
        RDEVEL , RTUSER , RA     )
!     -------------------------------------------------------------
!***********************************************************************

! defines the Boundary Conditions for the lagrangian module
!
!   GETFBR(CHAINE,NLELT,LSTELT) :
!   - CHAINE est une chaine de caractere fournie par l'utilisateur
!     qui donne les criteres de selection
!   - NLTELT est renvoye par la commande. C'est un entier qui
!     correspond au nombre de faces de bord trouveees repondant au
!     critere
!   - LSTELT est renvoye par la commande. C'est un tableau d'entiers
!     de taille NLTELT donnant la liste des faces de bord trouvees
!     repondant au critere.

!   CHAINE peut etre constitue de :
!   - references de couleurs (ex. : 1, 8, 26, ...
!   - references de groupes (ex. : entrees, groupe1, ...)
!   - criteres geometriques (ex. X<0.1, Y>=0.25, ...)
!   Ces criteres peuvent etre combines par des operateurs logiques
!   (AND et OR) et des parentheses
!   ex. : '1 AND (groupe2 OR groupe3) AND Y<1' permettra de recuperer
!   les faces de bord de couleur 1, appartenant aux groupes 'groupe2'
!   ou 'groupe3' et de coordonnee Y inferieure a 1.

!***********************************************************************
!
      IMPLICIT NONE
!
!***********************************************************************
!     DONNEES EN COMMON
!***********************************************************************
!
      INCLUDE "paramx.h"
      INCLUDE "numvar.h"
      INCLUDE "optcal.h"
      INCLUDE "cstnum.h"
      INCLUDE "cstphy.h"
      INCLUDE "entsor.h"
      INCLUDE "lagpar.h"
      INCLUDE "lagran.h"
      INCLUDE "ppppar.h"
      INCLUDE "ppthch.h"
      INCLUDE "cpincl.h"	
      include "ihmpre.h"
!
!***********************************************************************
!
! ARGUMENTS
!
      INTEGER          IDBIA0 , IDBRA0
      INTEGER          NDIM   , NCELET , NCEL   , NFAC   , NFABOR
      INTEGER          NFML   , NPRFML
      INTEGER          NNOD   , LNDFAC , LNDFBR , NCELBR
      INTEGER          NVAR   , NSCAL  , NPHAS
      INTEGER          NBPMAX , NVP    , NVP1   , NVEP  , NIVEP
      INTEGER          NTERSL , NVLSTA , NVISBR
      INTEGER          NIDEVE , NRDEVE , NITUSE , NRTUSE
!
      INTEGER          IFACEL(2,NFAC) , IFABOR(NFABOR)
      INTEGER          IFMFBR(NFABOR) , IFMCEL(NCELET)
      INTEGER          IPRFML(NFML,NPRFML)
      INTEGER          MAXELT, LSTELT(MAXELT)
      INTEGER          IPNFAC(NFAC+1) , NODFAC(LNDFAC)
      INTEGER          IPNFBR(NFABOR+1) , NODFBR(LNDFBR)
      INTEGER          ITYPFB(NFABOR,NPHAS) , ITRIFB(NFABOR,NPHAS)
      INTEGER          ITEPA(NBPMAX,NIVEP) , IFRLAG(NFABOR)
      INTEGER          IDEVEL(NIDEVE) , ITUSER(NITUSE)
      INTEGER          IA(*)
!
      DOUBLE PRECISION XYZCEN(NDIM,NCELET)
      DOUBLE PRECISION SURFAC(NDIM,NFAC) , SURFBO(NDIM,NFABOR)
      DOUBLE PRECISION CDGFAC(NDIM,NFAC) , CDGFBO(NDIM,NFABOR)
      DOUBLE PRECISION XYZNOD(NDIM,NNOD) , VOLUME(NCELET)
      DOUBLE PRECISION DT(NCELET) , RTPA(NCELET,*)
      DOUBLE PRECISION PROPCE(NCELET,*)
      DOUBLE PRECISION PROPFA(NFAC,*) , PROPFB(NFABOR,*)
      DOUBLE PRECISION COEFA(NFABOR,*) , COEFB(NFABOR,*)
      DOUBLE PRECISION ETTP(NBPMAX,NVP) , TEPA(NBPMAX,NVEP)
      DOUBLE PRECISION RDEVEL(NRDEVE) , RTUSER(NRTUSE)
      DOUBLE PRECISION RA(*)
!
! VARIABLES LOCALES
!
      INTEGER          IDEBIA, IDEBRA
      INTEGER          IFAC , IZONE, NBCLAS, ICLAS
      INTEGER          ICHA
      INTEGER          ILELT, NLELT
      INTEGER          MZONE

      INTEGER          I
      data I /0/
!
      DOUBLE PRECISION PIS6 , MP0 , TEMP, R
!
!***********************************************************************
!
!
!
!=======================================================================
! 1.  GESTION MEMOIRE
!=======================================================================
!
      IDEBIA = IDBIA0
      IDEBRA = IDBRA0

!
!=======================================================================
! 2. INITIALISATION
!=======================================================================
!
      PIS6 = PI/6.D0
!
!=======================================================================
! 3. Boundary Conditions
!=======================================================================
!
! IFRLAG(NFABOR): 
! In the lagrangian module, the user defines NFRLAG boundary zones 
! from the color of the boundary faces, or more generally from their properties 
! (colors, groups ...), from the boundary conditions defined in usclim, or even from 
! their coordinates. To do so, the array IFRLAG(NFABOR) giving for each face IFAC the 
! number IFRLAG(IFAC) corresponding to the zone to which it belongs, is completed. The zone 
! numbers (i.e. the values of IFRLAG(IFAC)) are chosen freely by the user, but must be strictly 
! positive integers inferior or equal to NFLAGM (parameter stored in lagpar.h, whose default 
! value is 100). A zone type is associated with every zone; it will be used to impose global 
! boundary conditions. WARNING: it is essential that every boundary face belongs to a zone..
!
! IUSNCL(NFLAGM): 
! For all the NFRLAG boundary zones previously identified, the number of classes 
! NBCLAS34 of entering particles is given: IUSNCL(IZONE) = NBCLAS. By default, the number of 
! particle classes is zero. The maximum number of classes is NCLAGM (parameter stored in 
! lagpar.h, whose default value is 20)..
! IUSCLB(NFLAGM): 
! For all the NFRLAG boundary zones previously identified, a particle boundary condition 
! type is given. There are two categories of particle boundary condition types: those 
! predefined in the subroutine uslabo 
! (marked out by the key words IENTRL, ISORTL, IREBOL, IDEPO1, IDEPO2, IDEPO3, IENCRL) and the 
! user boundary condition types (marked out by the key words JBORD1 to JBORD5), whose 
! corresponding particle behaviour must be defined in the subroutine uslabo.

     IZONE = -1
!
! ---> first zone -> walls
     IZONE = 1
     CALL GETFBR('all[]',NLELT,LSTELT)
     DO ILELT = 1, NLELT
       IFAC = LSTELT(ILELT)
       IFRLAG(IFAC) = IZONE
     ENDDO

! ---> second zone -> outlet
     IZONE =2 
     CALL GETFBR('OUTLET',NLELT,LSTELT)
     DO IFAC = 1, NFABOR
       IF(ITYPFB(IFAC,1).EQ.ISOLIB) THEN
         IFRLAG(IFAC) = IZONE
       ENDIF
     ENDDO
!
! ---> third zone ... -> inlet 
     IZONE = 3 
     CALL GETFBR('INJECT1',NLELT,LSTELT)
     DO ILELT = 1, NLELT
       IFAC = LSTELT(ILELT)
       IFRLAG(IFAC) = IZONE
     ENDDO

     IZONE = 3
     CALL GETFBR('INJECT2',NLELT,LSTELT)
     DO ILELT = 1, NLELT
       IFAC = LSTELT(ILELT)
       IFRLAG(IFAC) = IZONE
     ENDDO

!
!=======================================================================
! 4. Particle Injection
!=======================================================================
!
!   1) define the number of particle classes IUSNCL
!      default: IUSNCL = 0
!
!   2) define the physical properties and initial conditions
!      per inlet and per class
!
! ---> first zone -> walls no injection
      IZONE     = 1
      NBCLAS    = 0
      IUSNCL(IZONE) = NBCLAS
      IUSCLB(IZONE) = IDEPO1
!
! ---> second zone -> exit
      IZONE     = 2
      NBCLAS    = 0
      IUSNCL(IZONE) = NBCLAS
      IUSCLB(IZONE) = ISORTL
!
! ---> third zone -> inlet
      IZONE     = 3
      NBCLAS    = 1 
      IUSNCL(IZONE) = NBCLAS
      IUSCLB(IZONE) = IENTRL
!
!     IUSNCL number of classes per zone
!     IUSCLB boundary conditions regarding the particles
!     = IENTRL -> injection zone
!     = ISORTL -> outlet
!     = IREBOL -> rebound
!     = IDEPO1 -> deposition of particles (particles are treated as they
!                 leave the domain
!     = IDEPO2 -> deposition of particles, but particles stay in the
!                 calculation domain -> useful for postprocessing, but
!                 more memory is needed the in the case IDEPO1
!     = IDEPO3 -> deposition of particles, but in contrast to IDEPO2
!                 they could leave the surface again
!     = IDEPFA -> deposition de la particule avec force d'attachement,
!                 vitesse conservee et re-entrainement possible
!                 (Possible si LADLVO = 1 )
!     = IENCRL -> fouling (only coal module IPHYLA = 2)
!     = JBORD1 -> interaction part/frontiere utilisateur (cf. USLABO)
!     = JBORD2 -> interaction part/frontiere utilisateur (cf. USLABO)
!     = JBORD3 -> interaction part/frontiere utilisateur (cf. USLABO)
!     = JBORD4 -> interaction part/frontiere utilisateur (cf. USLABO)
!     = JBORD5 -> interaction part/frontiere utilisateur (cf. USLABO)
!
!
! USLAG(NCLAGM,NFLAGM,NDLAIM): Some pieces of information must be given for each 
! particle class associated with an injection zone. The first part consists in integers 
! contained in the array IUSLAG. There are at the most NDLAIM integers. These pieces of 
! information must be provided for each class ICLAS and each particle injection zone IZONE. 
! They are marked out by means of "pointers":
!  IUSLAG(ICLAS,IZONE,IJNBP): 
!                 number of particles to inject in the calculation domain per 
!                 class and per zone.
! IUSLAG(ICLAS,IZONE,IJFRE): injection period (expressed in number of time steps). 
!            If the period is null, then there is injection only at the first 
!            absolute lagrangian time step (including the restart calculations).
! IUSLAG(ICLAS,IZONE,IJUVW): type of velocity condition
!                            -1 particle injection vel. equal to the
!                            fluid velocity
!                             0 particle velocity normal to injection
!                             boundary with norm RUSLAG(ICLAS,IZONE,IUNO)
!                             1 particle velocity as given by RUSLAG()
! IUSLAG(ICLAS,IZONE,INUCHL): when the particles are coal particles (IPHYLA = 2), 
!                             this part of the array contains the coal 
!                             index-number, between 1 and NCHARB
!                             (defined by the user in the thermochemical file 
!                             dp_FCP, with NCHARB < NCHARM = 3).
!
!        IJPRTP : type of temperature boundary conditions
!                  =  1 forced temperature - RUSLAG(ICLAS,IZONE,ITPT)
!                  =  2 user supplied profile 
!        INUCHL : numero du charbon de la particule (uniquement si IPHYLA=2)
!
!     RUSLAG :
!        IUNO  : norm of velocity (m/s)
!        IUPT  : components of the particle injection velocity (m/s)
!        IVPT  : 
!        IWPT  : 
!        IDEBT : particle mass flow (kg/s)
!        IPOIT : statistical particle weight per class and zone
!                
!     En fonction de la physique
!        IDPT   : diameter (m)
!        IVDPT  : standard deviation of injection diameter (m)
!        ITPT   : temperature in C (Pas d'enthalpie)
!        ICPT   : specific heat capacity (J/kg/K)
!        IEPSI  : emissivity (si =0 if radiation module is active)
!        IROPT  : density (kg/m3)
!
!     Si Charbon (IPHYLA=2)
!        IHPT  : temperature in C  (Pas d'enthalpie)
!        IMCHT : mass of reactive coal (kg)
!        IMCKT : mass of coke (kg)
!
! IJNBP  - number of particles per class and per zone
! IJFRE  - injection period (number of time steps) ; 
!         = 0 injection only at first absolute time step
! ICLST  - statistical calculation enabled
! IJUVW  - type of velocity condition
!         = -1 particle velocity equals continuous phase velocity
!         = 0 the particle velocity is imposed perpendicular to the
!              injection boundary face and with the  norm
!              RUSLAG(ICLAS,IZONE,IUNO)
!         = 1  particle velocity is defined by RUSLAG(ICLAS,IZONE,IUPT)
! IJPRDP - type of diameter boundary conditions
!         = 1 predefined profile
!             RUSLAG(ICLAS,IZONE,IDPT),RUSLAG(ICLAS,IZONE,IVDPT)
!             IDPT - diameter
!             IVDPT - variance of diameter at injection
!             the diameter of each particle is calculated from the mean diameter,
!             the standard deviation and a gaussian random number
!             if particle diameter should vary it is strongly recommended to use
!             uslain.f90
!         = 2 user defined profile 
! IJPRTP - type of temperature initial conditions 
!         = 1 uniform temperature RUSLAG(ICLAS,IZONE,ITPT) degree C
!         = 2 user defined profile          
! IJPRPD - type of statistical weight condition
!         =1 uniform, defined by RUSLAG(ICLAS,IZONE,IPOIT)
!         =2 user defined, uslapr.f90

  DO IZONE  = 3,3
  NBCLAS = IUSNCL(IZONE)
  IUSCLB(IZONE) = IENTRL
  DO ICLAS = 1,NBCLAS
    IUSLAG(ICLAS,IZONE,IJNBP) = 0
    IUSLAG(ICLAS,IZONE,IJNBP) = 10
    IUSLAG(ICLAS,IZONE,IJFRE) = 1
!    if ((NTCABS .gt.20) .and. (NTCABS.lt.100)) then
!    IUSLAG(ICLAS,IZONE,IJFRE)   = 1
!    IUSLAG(ICLAS,IZONE,IJNBP)   = 10 
!    endif
! NBCLST is defined in uslag1.f90
    IF (NBCLST.GT.0) THEN
      IUSLAG(ICLAS,IZONE,ICLST) = 0
    ENDIF
    ! particle velocity
    IUSLAG(ICLAS,IZONE,IJUVW) = 1
    RUSLAG(ICLAS,IZONE,IUPT)  = 0.0D0
    RUSLAG(ICLAS,IZONE,IVPT)  = 5.0D0
    RUSLAG(ICLAS,IZONE,IWPT)  = 0.0D0

    ! diameter profile
    IUSLAG(ICLAS,IZONE,IJPRDP) = 1
    RUSLAG(ICLAS,IZONE,IDPT)   = 480.0D-6
    RUSLAG(ICLAS,IZONE,IVDPT)  = 0.0D1
    RUSLAG(ICLAS,IZONE,IROPT)  = 681.05

    ! temperature
    IUSLAG(ICLAS,IZONE,IJPRTP) = 1
    RUSLAG(ICLAS,IZONE,ITPT)   = 24.0d0

    ! statistical weights
    IUSLAG(ICLAS,IZONE,IJPRPD) = 1
    RUSLAG(ICLAS,IZONE,IPOIT)  = 1.0D0


    IF ( IPHYLA.EQ.1 ) THEN
      IF ( ITPVAR.EQ.1 ) THEN
    ! Temperature et Cp
      ENDIF
    ELSE IF ( IPHYLA.EQ.2 ) THEN
    ! coal combustion
	  ELSE IF (IPHYLA .eq. 3) THEN
    ! evaporation modell
      RUSLAG(ICLAS,IZONE,ICPT)    = 2.25D3
      RUSLAG(ICLAS,IZONE,IEPSI)   = 0.0D0
    ENDIF
  END DO
  END DO

!=======================================================================
!
!--------
! FORMATS
!--------
!
!----
! FIN
!----
!
      RETURN
!
      END


      
